@@ -1,6 +1,5 @@
 # -*- coding: utf-8 -*-
-
-# Copyright 2020 Google LLC
+# Copyright 2022 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,21 +13,19 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
+from typing import MutableMapping, MutableSequence
 
+from google.protobuf import field_mask_pb2  # type: ignore
 import proto  # type: ignore
 
-
-from google.cloud.datacatalog_v1beta1.types import common
 from google.cloud.datacatalog_v1beta1.types import (
     gcs_fileset_spec as gcd_gcs_fileset_spec,
 )
+from google.cloud.datacatalog_v1beta1.types import common
 from google.cloud.datacatalog_v1beta1.types import schema as gcd_schema
-from google.cloud.datacatalog_v1beta1.types import search
-from google.cloud.datacatalog_v1beta1.types import table_spec
+from google.cloud.datacatalog_v1beta1.types import search, table_spec
 from google.cloud.datacatalog_v1beta1.types import tags as gcd_tags
 from google.cloud.datacatalog_v1beta1.types import timestamps
-from google.protobuf import field_mask_pb2 as field_mask  # type: ignore
-
 
 __protobuf__ = proto.module(
     package="google.cloud.datacatalog.v1beta1",
@@ -132,13 +129,12 @@ class SearchCatalogRequest(proto.Message):
         matching.
 
         Attributes:
-            include_org_ids (Sequence[str]):
+            include_org_ids (MutableSequence[str]):
                 The list of organization IDs to search
                 within. To find your organization ID, follow
                 instructions in
-                https://cloud.google.com/resource-
-                manager/docs/creating-managing-organization.
-            include_project_ids (Sequence[str]):
+                https://cloud.google.com/resource-manager/docs/creating-managing-organization.
+            include_project_ids (MutableSequence[str]):
                 The list of project IDs to search within. To
                 learn more about the distinction between project
                 names/IDs/numbers, go to
@@ -150,21 +146,40 @@ class SearchCatalogRequest(proto.Message):
                 By default, GCP public datasets are excluded.
         """
 
-        include_org_ids = proto.RepeatedField(proto.STRING, number=2)
+        include_org_ids: MutableSequence[str] = proto.RepeatedField(
+            proto.STRING,
+            number=2,
+        )
+        include_project_ids: MutableSequence[str] = proto.RepeatedField(
+            proto.STRING,
+            number=3,
+        )
+        include_gcp_public_datasets: bool = proto.Field(
+            proto.BOOL,
+            number=7,
+        )
 
-        include_project_ids = proto.RepeatedField(proto.STRING, number=3)
-
-        include_gcp_public_datasets = proto.Field(proto.BOOL, number=7)
-
-    scope = proto.Field(proto.MESSAGE, number=6, message=Scope,)
-
-    query = proto.Field(proto.STRING, number=1)
-
-    page_size = proto.Field(proto.INT32, number=2)
-
-    page_token = proto.Field(proto.STRING, number=3)
-
-    order_by = proto.Field(proto.STRING, number=5)
+    scope: Scope = proto.Field(
+        proto.MESSAGE,
+        number=6,
+        message=Scope,
+    )
+    query: str = proto.Field(
+        proto.STRING,
+        number=1,
+    )
+    page_size: int = proto.Field(
+        proto.INT32,
+        number=2,
+    )
+    page_token: str = proto.Field(
+        proto.STRING,
+        number=3,
+    )
+    order_by: str = proto.Field(
+        proto.STRING,
+        number=5,
+    )
 
 
 class SearchCatalogResponse(proto.Message):
@@ -172,7 +187,7 @@ class SearchCatalogResponse(proto.Message):
     [SearchCatalog][google.cloud.datacatalog.v1beta1.DataCatalog.SearchCatalog].
 
     Attributes:
-        results (Sequence[google.cloud.datacatalog_v1beta1.types.SearchCatalogResult]):
+        results (MutableSequence[google.cloud.datacatalog_v1beta1.types.SearchCatalogResult]):
             Search results.
         next_page_token (str):
             The token that can be used to retrieve the
@@ -183,11 +198,15 @@ class SearchCatalogResponse(proto.Message):
     def raw_page(self):
         return self
 
-    results = proto.RepeatedField(
-        proto.MESSAGE, number=1, message=search.SearchCatalogResult,
+    results: MutableSequence[search.SearchCatalogResult] = proto.RepeatedField(
+        proto.MESSAGE,
+        number=1,
+        message=search.SearchCatalogResult,
+    )
+    next_page_token: str = proto.Field(
+        proto.STRING,
+        number=3,
     )
-
-    next_page_token = proto.Field(proto.STRING, number=3)
 
 
 class CreateEntryGroupRequest(proto.Message):
@@ -214,11 +233,19 @@ class CreateEntryGroupRequest(proto.Message):
             empty entry group.
     """
 
-    parent = proto.Field(proto.STRING, number=1)
-
-    entry_group_id = proto.Field(proto.STRING, number=3)
-
-    entry_group = proto.Field(proto.MESSAGE, number=2, message="EntryGroup",)
+    parent: str = proto.Field(
+        proto.STRING,
+        number=1,
+    )
+    entry_group_id: str = proto.Field(
+        proto.STRING,
+        number=3,
+    )
+    entry_group: "EntryGroup" = proto.Field(
+        proto.MESSAGE,
+        number=2,
+        message="EntryGroup",
+    )
 
 
 class UpdateEntryGroupRequest(proto.Message):
@@ -235,9 +262,16 @@ class UpdateEntryGroupRequest(proto.Message):
             updated.
     """
 
-    entry_group = proto.Field(proto.MESSAGE, number=1, message="EntryGroup",)
-
-    update_mask = proto.Field(proto.MESSAGE, number=2, message=field_mask.FieldMask,)
+    entry_group: "EntryGroup" = proto.Field(
+        proto.MESSAGE,
+        number=1,
+        message="EntryGroup",
+    )
+    update_mask: field_mask_pb2.FieldMask = proto.Field(
+        proto.MESSAGE,
+        number=2,
+        message=field_mask_pb2.FieldMask,
+    )
 
 
 class GetEntryGroupRequest(proto.Message):
@@ -253,9 +287,15 @@ class GetEntryGroupRequest(proto.Message):
             all fields are returned.
     """
 
-    name = proto.Field(proto.STRING, number=1)
-
-    read_mask = proto.Field(proto.MESSAGE, number=2, message=field_mask.FieldMask,)
+    name: str = proto.Field(
+        proto.STRING,
+        number=1,
+    )
+    read_mask: field_mask_pb2.FieldMask = proto.Field(
+        proto.MESSAGE,
+        number=2,
+        message=field_mask_pb2.FieldMask,
+    )
 
 
 class DeleteEntryGroupRequest(proto.Message):
@@ -271,9 +311,14 @@ class DeleteEntryGroupRequest(proto.Message):
             entry group.
     """
 
-    name = proto.Field(proto.STRING, number=1)
-
-    force = proto.Field(proto.BOOL, number=2)
+    name: str = proto.Field(
+        proto.STRING,
+        number=1,
+    )
+    force: bool = proto.Field(
+        proto.BOOL,
+        number=2,
+    )
 
 
 class ListEntryGroupsRequest(proto.Message):
@@ -295,11 +340,18 @@ class ListEntryGroupsRequest(proto.Message):
             requested. If empty, the first page is returned.
     """
 
-    parent = proto.Field(proto.STRING, number=1)
-
-    page_size = proto.Field(proto.INT32, number=2)
-
-    page_token = proto.Field(proto.STRING, number=3)
+    parent: str = proto.Field(
+        proto.STRING,
+        number=1,
+    )
+    page_size: int = proto.Field(
+        proto.INT32,
+        number=2,
+    )
+    page_token: str = proto.Field(
+        proto.STRING,
+        number=3,
+    )
 
 
 class ListEntryGroupsResponse(proto.Message):
@@ -307,7 +359,7 @@ class ListEntryGroupsResponse(proto.Message):
     [ListEntryGroups][google.cloud.datacatalog.v1beta1.DataCatalog.ListEntryGroups].
 
     Attributes:
-        entry_groups (Sequence[google.cloud.datacatalog_v1beta1.types.EntryGroup]):
+        entry_groups (MutableSequence[google.cloud.datacatalog_v1beta1.types.EntryGroup]):
             EntryGroup details.
         next_page_token (str):
             Token to retrieve the next page of results.
@@ -319,9 +371,15 @@ class ListEntryGroupsResponse(proto.Message):
     def raw_page(self):
         return self
 
-    entry_groups = proto.RepeatedField(proto.MESSAGE, number=1, message="EntryGroup",)
-
-    next_page_token = proto.Field(proto.STRING, number=2)
+    entry_groups: MutableSequence["EntryGroup"] = proto.RepeatedField(
+        proto.MESSAGE,
+        number=1,
+        message="EntryGroup",
+    )
+    next_page_token: str = proto.Field(
+        proto.STRING,
+        number=2,
+    )
 
 
 class CreateEntryRequest(proto.Message):
@@ -343,11 +401,19 @@ class CreateEntryRequest(proto.Message):
             Required. The entry to create.
     """
 
-    parent = proto.Field(proto.STRING, number=1)
-
-    entry_id = proto.Field(proto.STRING, number=3)
-
-    entry = proto.Field(proto.MESSAGE, number=2, message="Entry",)
+    parent: str = proto.Field(
+        proto.STRING,
+        number=1,
+    )
+    entry_id: str = proto.Field(
+        proto.STRING,
+        number=3,
+    )
+    entry: "Entry" = proto.Field(
+        proto.MESSAGE,
+        number=2,
+        message="Entry",
+    )
 
 
 class UpdateEntryRequest(proto.Message):
@@ -387,9 +453,16 @@ class UpdateEntryRequest(proto.Message):
                -  source_system_timestamps
     """
 
-    entry = proto.Field(proto.MESSAGE, number=1, message="Entry",)
-
-    update_mask = proto.Field(proto.MESSAGE, number=2, message=field_mask.FieldMask,)
+    entry: "Entry" = proto.Field(
+        proto.MESSAGE,
+        number=1,
+        message="Entry",
+    )
+    update_mask: field_mask_pb2.FieldMask = proto.Field(
+        proto.MESSAGE,
+        number=2,
+        message=field_mask_pb2.FieldMask,
+    )
 
 
 class DeleteEntryRequest(proto.Message):
@@ -403,7 +476,10 @@ class DeleteEntryRequest(proto.Message):
             -  projects/{project_id}/locations/{location}/entryGroups/{entry_group_id}/entries/{entry_id}
     """
 
-    name = proto.Field(proto.STRING, number=1)
+    name: str = proto.Field(
+        proto.STRING,
+        number=1,
+    )
 
 
 class GetEntryRequest(proto.Message):
@@ -417,13 +493,23 @@ class GetEntryRequest(proto.Message):
             -  projects/{project_id}/locations/{location}/entryGroups/{entry_group_id}/entries/{entry_id}
     """
 
-    name = proto.Field(proto.STRING, number=1)
+    name: str = proto.Field(
+        proto.STRING,
+        number=1,
+    )
 
 
 class LookupEntryRequest(proto.Message):
     r"""Request message for
     [LookupEntry][google.cloud.datacatalog.v1beta1.DataCatalog.LookupEntry].
 
+    This message has `oneof`_ fields (mutually exclusive fields).
+    For each oneof, at most one member field can be set at the same time.
+    Setting any member of the oneof automatically clears all other
+    members.
+
+    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields
+
     Attributes:
         linked_resource (str):
             The full name of the Google Cloud Platform resource the Data
@@ -435,6 +521,8 @@ class LookupEntryRequest(proto.Message):
 
             -  //bigquery.googleapis.com/projects/projectId/datasets/datasetId/tables/tableId
             -  //pubsub.googleapis.com/projects/projectId/topics/topicId
+
+            This field is a member of `oneof`_ ``target_name``.
         sql_resource (str):
             The SQL name of the entry. SQL names are case-sensitive.
 
@@ -449,11 +537,20 @@ class LookupEntryRequest(proto.Message):
             ``*_id``\ s shoud satisfy the standard SQL rules for
             identifiers.
             https://cloud.google.com/bigquery/docs/reference/standard-sql/lexical.
+
+            This field is a member of `oneof`_ ``target_name``.
     """
 
-    linked_resource = proto.Field(proto.STRING, number=1, oneof="target_name")
-
-    sql_resource = proto.Field(proto.STRING, number=3, oneof="target_name")
+    linked_resource: str = proto.Field(
+        proto.STRING,
+        number=1,
+        oneof="target_name",
+    )
+    sql_resource: str = proto.Field(
+        proto.STRING,
+        number=3,
+        oneof="target_name",
+    )
 
 
 class Entry(proto.Message):
@@ -467,6 +564,13 @@ class Entry(proto.Message):
     Entry can also be used to attach flexible metadata, such as a
     [Tag][google.cloud.datacatalog.v1beta1.Tag].
 
+    This message has `oneof`_ fields (mutually exclusive fields).
+    For each oneof, at most one member field can be set at the same time.
+    Setting any member of the oneof automatically clears all other
+    members.
+
+    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields
+
     Attributes:
         name (str):
             The Data Catalog resource name of the entry in URL format.
@@ -494,6 +598,8 @@ class Entry(proto.Message):
             The type of the entry.
             Only used for Entries with types in the
             EntryType enum.
+
+            This field is a member of `oneof`_ ``entry_type``.
         user_specified_type (str):
             Entry type if it does not fit any of the input-allowed
             values listed in ``EntryType`` enum above. When creating an
@@ -508,10 +614,14 @@ class Entry(proto.Message):
             Currently, only FILESET enum value is allowed. All other
             entries created through Data Catalog must use
             ``user_specified_type``.
+
+            This field is a member of `oneof`_ ``entry_type``.
         integrated_system (google.cloud.datacatalog_v1beta1.types.IntegratedSystem):
             Output only. This field indicates the entry's
             source system that Data Catalog integrates with,
             such as BigQuery or Pub/Sub.
+
+            This field is a member of `oneof`_ ``system``.
         user_specified_system (str):
             This field indicates the entry's source system that Data
             Catalog does not integrate with. ``user_specified_system``
@@ -519,17 +629,25 @@ class Entry(proto.Message):
             contain letters, numbers, and underscores; are case
             insensitive; must be at least 1 character and at most 64
             characters long.
+
+            This field is a member of `oneof`_ ``system``.
         gcs_fileset_spec (google.cloud.datacatalog_v1beta1.types.GcsFilesetSpec):
             Specification that applies to a Cloud Storage
             fileset. This is only valid on entries of type
             FILESET.
+
+            This field is a member of `oneof`_ ``type_spec``.
         bigquery_table_spec (google.cloud.datacatalog_v1beta1.types.BigQueryTableSpec):
             Specification that applies to a BigQuery table. This is only
             valid on entries of type ``TABLE``.
+
+            This field is a member of `oneof`_ ``type_spec``.
         bigquery_date_sharded_spec (google.cloud.datacatalog_v1beta1.types.BigQueryDateShardedSpec):
             Specification for a group of BigQuery tables with name
             pattern ``[prefix]YYYYMMDD``. Context:
             https://cloud.google.com/bigquery/docs/partitioned-tables#partitioning_versus_sharding.
+
+            This field is a member of `oneof`_ ``type_spec``.
         display_name (str):
             Display information such as title and
             description. A short name to identify the entry,
@@ -551,49 +669,71 @@ class Entry(proto.Message):
             an empty timestamp.
     """
 
-    name = proto.Field(proto.STRING, number=1)
-
-    linked_resource = proto.Field(proto.STRING, number=9)
-
-    type_ = proto.Field(proto.ENUM, number=2, oneof="entry_type", enum="EntryType",)
-
-    user_specified_type = proto.Field(proto.STRING, number=16, oneof="entry_type")
-
-    integrated_system = proto.Field(
-        proto.ENUM, number=17, oneof="system", enum=common.IntegratedSystem,
+    name: str = proto.Field(
+        proto.STRING,
+        number=1,
     )
-
-    user_specified_system = proto.Field(proto.STRING, number=18, oneof="system")
-
-    gcs_fileset_spec = proto.Field(
+    linked_resource: str = proto.Field(
+        proto.STRING,
+        number=9,
+    )
+    type_: "EntryType" = proto.Field(
+        proto.ENUM,
+        number=2,
+        oneof="entry_type",
+        enum="EntryType",
+    )
+    user_specified_type: str = proto.Field(
+        proto.STRING,
+        number=16,
+        oneof="entry_type",
+    )
+    integrated_system: common.IntegratedSystem = proto.Field(
+        proto.ENUM,
+        number=17,
+        oneof="system",
+        enum=common.IntegratedSystem,
+    )
+    user_specified_system: str = proto.Field(
+        proto.STRING,
+        number=18,
+        oneof="system",
+    )
+    gcs_fileset_spec: gcd_gcs_fileset_spec.GcsFilesetSpec = proto.Field(
         proto.MESSAGE,
         number=6,
         oneof="type_spec",
         message=gcd_gcs_fileset_spec.GcsFilesetSpec,
     )
-
-    bigquery_table_spec = proto.Field(
+    bigquery_table_spec: table_spec.BigQueryTableSpec = proto.Field(
         proto.MESSAGE,
         number=12,
         oneof="type_spec",
         message=table_spec.BigQueryTableSpec,
     )
-
-    bigquery_date_sharded_spec = proto.Field(
+    bigquery_date_sharded_spec: table_spec.BigQueryDateShardedSpec = proto.Field(
         proto.MESSAGE,
         number=15,
         oneof="type_spec",
         message=table_spec.BigQueryDateShardedSpec,
     )
-
-    display_name = proto.Field(proto.STRING, number=3)
-
-    description = proto.Field(proto.STRING, number=4)
-
-    schema = proto.Field(proto.MESSAGE, number=5, message=gcd_schema.Schema,)
-
-    source_system_timestamps = proto.Field(
-        proto.MESSAGE, number=7, message=timestamps.SystemTimestamps,
+    display_name: str = proto.Field(
+        proto.STRING,
+        number=3,
+    )
+    description: str = proto.Field(
+        proto.STRING,
+        number=4,
+    )
+    schema: gcd_schema.Schema = proto.Field(
+        proto.MESSAGE,
+        number=5,
+        message=gcd_schema.Schema,
+    )
+    source_system_timestamps: timestamps.SystemTimestamps = proto.Field(
+        proto.MESSAGE,
+        number=7,
+        message=timestamps.SystemTimestamps,
     )
 
 
@@ -624,14 +764,22 @@ class EntryGroup(proto.Message):
             EntryGroup. Default value is empty timestamps.
     """
 
-    name = proto.Field(proto.STRING, number=1)
-
-    display_name = proto.Field(proto.STRING, number=2)
-
-    description = proto.Field(proto.STRING, number=3)
-
-    data_catalog_timestamps = proto.Field(
-        proto.MESSAGE, number=4, message=timestamps.SystemTimestamps,
+    name: str = proto.Field(
+        proto.STRING,
+        number=1,
+    )
+    display_name: str = proto.Field(
+        proto.STRING,
+        number=2,
+    )
+    description: str = proto.Field(
+        proto.STRING,
+        number=3,
+    )
+    data_catalog_timestamps: timestamps.SystemTimestamps = proto.Field(
+        proto.MESSAGE,
+        number=4,
+        message=timestamps.SystemTimestamps,
     )
 
 
@@ -654,11 +802,19 @@ class CreateTagTemplateRequest(proto.Message):
             Required. The tag template to create.
     """
 
-    parent = proto.Field(proto.STRING, number=1)
-
-    tag_template_id = proto.Field(proto.STRING, number=3)
-
-    tag_template = proto.Field(proto.MESSAGE, number=2, message=gcd_tags.TagTemplate,)
+    parent: str = proto.Field(
+        proto.STRING,
+        number=1,
+    )
+    tag_template_id: str = proto.Field(
+        proto.STRING,
+        number=3,
+    )
+    tag_template: gcd_tags.TagTemplate = proto.Field(
+        proto.MESSAGE,
+        number=2,
+        message=gcd_tags.TagTemplate,
+    )
 
 
 class GetTagTemplateRequest(proto.Message):
@@ -672,7 +828,10 @@ class GetTagTemplateRequest(proto.Message):
             -  projects/{project_id}/locations/{location}/tagTemplates/{tag_template_id}
     """
 
-    name = proto.Field(proto.STRING, number=1)
+    name: str = proto.Field(
+        proto.STRING,
+        number=1,
+    )
 
 
 class UpdateTagTemplateRequest(proto.Message):
@@ -695,9 +854,16 @@ class UpdateTagTemplateRequest(proto.Message):
             updated.
     """
 
-    tag_template = proto.Field(proto.MESSAGE, number=1, message=gcd_tags.TagTemplate,)
-
-    update_mask = proto.Field(proto.MESSAGE, number=2, message=field_mask.FieldMask,)
+    tag_template: gcd_tags.TagTemplate = proto.Field(
+        proto.MESSAGE,
+        number=1,
+        message=gcd_tags.TagTemplate,
+    )
+    update_mask: field_mask_pb2.FieldMask = proto.Field(
+        proto.MESSAGE,
+        number=2,
+        message=field_mask_pb2.FieldMask,
+    )
 
 
 class DeleteTagTemplateRequest(proto.Message):
@@ -716,9 +882,14 @@ class DeleteTagTemplateRequest(proto.Message):
             the future.
     """
 
-    name = proto.Field(proto.STRING, number=1)
-
-    force = proto.Field(proto.BOOL, number=2)
+    name: str = proto.Field(
+        proto.STRING,
+        number=1,
+    )
+    force: bool = proto.Field(
+        proto.BOOL,
+        number=2,
+    )
 
 
 class CreateTagRequest(proto.Message):
@@ -738,9 +909,15 @@ class CreateTagRequest(proto.Message):
             Required. The tag to create.
     """
 
-    parent = proto.Field(proto.STRING, number=1)
-
-    tag = proto.Field(proto.MESSAGE, number=2, message=gcd_tags.Tag,)
+    parent: str = proto.Field(
+        proto.STRING,
+        number=1,
+    )
+    tag: gcd_tags.Tag = proto.Field(
+        proto.MESSAGE,
+        number=2,
+        message=gcd_tags.Tag,
+    )
 
 
 class UpdateTagRequest(proto.Message):
@@ -757,9 +934,16 @@ class UpdateTagRequest(proto.Message):
             field is the field ``fields``.
     """
 
-    tag = proto.Field(proto.MESSAGE, number=1, message=gcd_tags.Tag,)
-
-    update_mask = proto.Field(proto.MESSAGE, number=2, message=field_mask.FieldMask,)
+    tag: gcd_tags.Tag = proto.Field(
+        proto.MESSAGE,
+        number=1,
+        message=gcd_tags.Tag,
+    )
+    update_mask: field_mask_pb2.FieldMask = proto.Field(
+        proto.MESSAGE,
+        number=2,
+        message=field_mask_pb2.FieldMask,
+    )
 
 
 class DeleteTagRequest(proto.Message):
@@ -773,7 +957,10 @@ class DeleteTagRequest(proto.Message):
             -  projects/{project_id}/locations/{location}/entryGroups/{entry_group_id}/entries/{entry_id}/tags/{tag_id}
     """
 
-    name = proto.Field(proto.STRING, number=1)
+    name: str = proto.Field(
+        proto.STRING,
+        number=1,
+    )
 
 
 class CreateTagTemplateFieldRequest(proto.Message):
@@ -798,12 +985,18 @@ class CreateTagTemplateFieldRequest(proto.Message):
             Required. The tag template field to create.
     """
 
-    parent = proto.Field(proto.STRING, number=1)
-
-    tag_template_field_id = proto.Field(proto.STRING, number=2)
-
-    tag_template_field = proto.Field(
-        proto.MESSAGE, number=3, message=gcd_tags.TagTemplateField,
+    parent: str = proto.Field(
+        proto.STRING,
+        number=1,
+    )
+    tag_template_field_id: str = proto.Field(
+        proto.STRING,
+        number=2,
+    )
+    tag_template_field: gcd_tags.TagTemplateField = proto.Field(
+        proto.MESSAGE,
+        number=3,
+        message=gcd_tags.TagTemplateField,
     )
 
 
@@ -836,13 +1029,20 @@ class UpdateTagTemplateFieldRequest(proto.Message):
             is NOT allowed.
     """
 
-    name = proto.Field(proto.STRING, number=1)
-
-    tag_template_field = proto.Field(
-        proto.MESSAGE, number=2, message=gcd_tags.TagTemplateField,
+    name: str = proto.Field(
+        proto.STRING,
+        number=1,
+    )
+    tag_template_field: gcd_tags.TagTemplateField = proto.Field(
+        proto.MESSAGE,
+        number=2,
+        message=gcd_tags.TagTemplateField,
+    )
+    update_mask: field_mask_pb2.FieldMask = proto.Field(
+        proto.MESSAGE,
+        number=3,
+        message=field_mask_pb2.FieldMask,
     )
-
-    update_mask = proto.Field(proto.MESSAGE, number=3, message=field_mask.FieldMask,)
 
 
 class RenameTagTemplateFieldRequest(proto.Message):
@@ -859,9 +1059,14 @@ class RenameTagTemplateFieldRequest(proto.Message):
             example, ``my_new_field``.
     """
 
-    name = proto.Field(proto.STRING, number=1)
-
-    new_tag_template_field_id = proto.Field(proto.STRING, number=2)
+    name: str = proto.Field(
+        proto.STRING,
+        number=1,
+    )
+    new_tag_template_field_id: str = proto.Field(
+        proto.STRING,
+        number=2,
+    )
 
 
 class DeleteTagTemplateFieldRequest(proto.Message):
@@ -881,9 +1086,14 @@ class DeleteTagTemplateFieldRequest(proto.Message):
             in the future.
     """
 
-    name = proto.Field(proto.STRING, number=1)
-
-    force = proto.Field(proto.BOOL, number=2)
+    name: str = proto.Field(
+        proto.STRING,
+        number=1,
+    )
+    force: bool = proto.Field(
+        proto.BOOL,
+        number=2,
+    )
 
 
 class ListTagsRequest(proto.Message):
@@ -909,11 +1119,18 @@ class ListTagsRequest(proto.Message):
             If empty, the first page is returned.
     """
 
-    parent = proto.Field(proto.STRING, number=1)
-
-    page_size = proto.Field(proto.INT32, number=2)
-
-    page_token = proto.Field(proto.STRING, number=3)
+    parent: str = proto.Field(
+        proto.STRING,
+        number=1,
+    )
+    page_size: int = proto.Field(
+        proto.INT32,
+        number=2,
+    )
+    page_token: str = proto.Field(
+        proto.STRING,
+        number=3,
+    )
 
 
 class ListTagsResponse(proto.Message):
@@ -921,7 +1138,7 @@ class ListTagsResponse(proto.Message):
     [ListTags][google.cloud.datacatalog.v1beta1.DataCatalog.ListTags].
 
     Attributes:
-        tags (Sequence[google.cloud.datacatalog_v1beta1.types.Tag]):
+        tags (MutableSequence[google.cloud.datacatalog_v1beta1.types.Tag]):
             [Tag][google.cloud.datacatalog.v1beta1.Tag] details.
         next_page_token (str):
             Token to retrieve the next page of results.
@@ -933,9 +1150,15 @@ class ListTagsResponse(proto.Message):
     def raw_page(self):
         return self
 
-    tags = proto.RepeatedField(proto.MESSAGE, number=1, message=gcd_tags.Tag,)
-
-    next_page_token = proto.Field(proto.STRING, number=2)
+    tags: MutableSequence[gcd_tags.Tag] = proto.RepeatedField(
+        proto.MESSAGE,
+        number=1,
+        message=gcd_tags.Tag,
+    )
+    next_page_token: str = proto.Field(
+        proto.STRING,
+        number=2,
+    )
 
 
 class ListEntriesRequest(proto.Message):
@@ -962,13 +1185,23 @@ class ListEntriesRequest(proto.Message):
             return a list of Entries with only "name" field.
     """
 
-    parent = proto.Field(proto.STRING, number=1)
-
-    page_size = proto.Field(proto.INT32, number=2)
-
-    page_token = proto.Field(proto.STRING, number=3)
-
-    read_mask = proto.Field(proto.MESSAGE, number=4, message=field_mask.FieldMask,)
+    parent: str = proto.Field(
+        proto.STRING,
+        number=1,
+    )
+    page_size: int = proto.Field(
+        proto.INT32,
+        number=2,
+    )
+    page_token: str = proto.Field(
+        proto.STRING,
+        number=3,
+    )
+    read_mask: field_mask_pb2.FieldMask = proto.Field(
+        proto.MESSAGE,
+        number=4,
+        message=field_mask_pb2.FieldMask,
+    )
 
 
 class ListEntriesResponse(proto.Message):
@@ -976,7 +1209,7 @@ class ListEntriesResponse(proto.Message):
     [ListEntries][google.cloud.datacatalog.v1beta1.DataCatalog.ListEntries].
 
     Attributes:
-        entries (Sequence[google.cloud.datacatalog_v1beta1.types.Entry]):
+        entries (MutableSequence[google.cloud.datacatalog_v1beta1.types.Entry]):
             Entry details.
         next_page_token (str):
             Token to retrieve the next page of results.
@@ -988,9 +1221,15 @@ class ListEntriesResponse(proto.Message):
     def raw_page(self):
         return self
 
-    entries = proto.RepeatedField(proto.MESSAGE, number=1, message="Entry",)
-
-    next_page_token = proto.Field(proto.STRING, number=2)
+    entries: MutableSequence["Entry"] = proto.RepeatedField(
+        proto.MESSAGE,
+        number=1,
+        message="Entry",
+    )
+    next_page_token: str = proto.Field(
+        proto.STRING,
+        number=2,
+    )
 
 
 __all__ = tuple(sorted(__protobuf__.manifest))
